```
from openpyxl import load_workbook
import random
import string

def OACD_read(filename): # put in OACD design and return combination compound and their levels
    fertnames = []
    combinations = dict()
    for i in range(1, 60):
        t = 'Comb_' + str(i) # combination numbering
        combinations[t] = []

    wb = load_workbook(filename)
    ws = wb['OACD'] # OACD design

    cell_range = ws['A1':'H1']
    cell_range = cell_range[0]
    for cell in cell_range:
        fertnames.append(cell.value)

    for i in range(2,61):
        t = 'Comb_' + str(i-1)
        j = -1
        cell_range = ws['A'+str(i):'H'+str(i)]
        cell_range = cell_range[0]
        for cell in cell_range:
            j += 1
            if cell.value != -1:
                combinations[t].append(fertnames[j] + '_' + str(cell.value))

    wb.close()

    result = [fertnames, combinations]
    return result

def Allocation(filename):
    allocation = dict()  # dictionary store gardening machine pod & combs pairs
    stack_1 = list()  # three lists help randomization
    stack_2 = list()
    temp = list()

    # stack_1 stores the triplicates of 59 combinations
    for i in range(1, 60):
        t = []
        t.append('Comb_' + str(i))
        t *= 3
        stack_1.append(t)

    # stack_1 stores triplicates of controls and traditional fertilizers
    stack_1.append(['Control'] * 3)
    stack_1.append(['Traditional'] * 3)

    wb = load_workbook(filename)  # file for OACD design
    ws = wb['Sheet1']  # sheet containing OACD design

    # stack_1 stores mono compound treatments with triplicates
    cell_range = ws['A1':'H1']
    cell_range = cell_range[0]
    for cell in cell_range:
        t = []
        t.append(cell.value + '_0')
        t *= 3
        stack_1.append(t)
        t = []
        t.append(cell.value + '_1')
        t *= 3
        stack_1.append(t)


    random.shuffle(stack_1)  # randomization


    # pairing gardening machines with compound treatments
    for letter in string.ascii_uppercase:


        car = list()  # temporary list for randomization

        if len(stack_1) < 9:
            pass
            if not stack_2:
                for j in stack_1:
                    car.append(j.pop())
                allocation['Z'] = car

            else:
                for j in stack_1:
                    car.append(j.pop())

                random.shuffle(stack_2)
                for j in range(9 - len(stack_1)):
                    t = stack_2.pop(0)
                    car.append(t.pop())
                    if t:
                        temp.append(t)

                stack_1 += stack_2
                random.shuffle(stack_1)
                stack_2 = temp
                temp = []

                allocation[letter] = car

        else:
            pass
            for i in range(9):
                t = stack_1.pop(0)
                car.append(t.pop())
                if t:
                    stack_2.append(t)

            allocation[letter] = car

        pass
        print('machine ' + letter + 'loading combinations completed！！！')
        print(allocation)  # print out results

    wb.close()

    # file store the randomization results
    with open('allocation.txt', 'w', encoding='utf-8') as f:
        for letter in string.ascii_uppercase:
            f.write(letter + ': ' + str(allocation[letter]))
            f.write('\n')

    return allocation


def Application(fertnames, combinations, allocation):

    application = dict()
    for fert in fertnames:
        for i in range(2):
            application[fert + '_' + str(i)] = []
    application['Traditional'] = []

    keys = application.keys()
    comb_names = combinations.keys()

    for letter in string.ascii_uppercase:
        i = 0
        for pod in allocation[letter]:
            i += 1
            if pod in keys:
                application[pod].append(letter + str(i)) 
            elif pod in comb_names:
                for fert in combinations[pod]:
                    application[fert].append(letter + str(i))

    with open('application.txt', 'w', encoding='utf-8') as f:
        for key in keys:
            f.write(key + ': ' + str(application[key]))
            f.write('\n')

    return application

if __name__ == "__main__":
    [fernames, combinations] = OACD_read('WisDM Green Experiment Design.xlsx')
    allocation = Allocation('WisDM Green Experiment Design.xlsx')
    application = Application(fernames, combinations, allocation)
    print(application)



from openpyxl import load_workbook
import random

def OACD_read(filename): # return fertilizer name list & combination level dictionary
    fertnames = []
    combinations = dict()
    for i in range(1, 37):
        t = 'Comb_' + str(i) # combination numbering
        combinations[t] = []

    wb = load_workbook(filename)
    ws = wb['Validation Combos']

    cell_range = ws['B1':'F1']
    cell_range = cell_range[0]
    for cell in cell_range:
        fertnames.append(cell.value)

    for i in range(2,38):
        t = 'Comb_' + str(i-1)
        j = -1
        cell_range = ws['B'+str(i):'F'+str(i)]
        cell_range = cell_range[0]
        for cell in cell_range:
            j += 1
            if cell.value != -1:
                combinations[t].append(fertnames[j] + '_' + str(cell.value))

    wb.close()

    result = [fertnames, combinations]
    return result

def Allocation(filename):
    allocation = dict()  # dictionary for pairing gardening machine pods with combinations
    stack_1 = list() # three lists help randomization
    stack_2 = list()
    temp = list()

    # stack_1 stores the triplicates of all combinations
    for i in range(1, 37):
        t = []  # temporary list for randomization
        t.append('Comb_' + str(i))
        t *= 3
        stack_1.append(t)



    # stack_1 stores controls
    stack_1.append(['Control'] * 3)
    stack_1.append(['Control'] * 3)


    # stack_1 stores mono_treatments of level 1 & 2
    stack_1.append(['Humic Acid_0'] * 3)
    stack_1.append(['Humic Acid_1'] * 3)
    stack_1.append(['Humic Acid_2'] * 3)
    stack_1.append(['EDTA-Fe_0'] * 3)
    stack_1.append(['EDTA-Fe_1'] * 3)
    stack_1.append(['EDTA-Fe_2'] * 3)
    stack_1.append(['Adenine_0'] * 3)
    stack_1.append(['Adenine_1'] * 3)
    stack_1.append(['Adenine_2'] * 3)
    stack_1.append(['6-BAP_0'] * 3)
    stack_1.append(['6-BAP_1'] * 3)
    stack_1.append(['6-BAP_2'] * 3)
    stack_1.append(['Seaweed_0'] * 3)
    stack_1.append(['Seaweed_1'] * 3)
    stack_1.append(['Seaweed_2'] * 3)

random.shuffle(stack_1)  # randomization

    # pair gardening machine pods with combs
    for i in range(ord('A'), ord('R')+1):
        letter = chr(i)

        car = list()  # temporary list for randomization

        if len(stack_1) < 9:
            pass
            if not stack_2:
                for j in stack_1:
                    car.append(j.pop())
                allocation['R'] = car

            else:
                if stack_1:
                    for j in stack_1:
                        car.append(j.pop())

                random.shuffle(stack_2)
                for j in range(9 - len(stack_1)):
                    if stack_2:
                        t = stack_2.pop(0)
                        car.append(t.pop())
                        if t:
                            temp.append(t)

                stack_1 += stack_2
                random.shuffle(stack_1)
                stack_2 = temp
                temp = []

                allocation[letter] = car

        else:
            pass
            for i in range(9):
                t = stack_1.pop(0)
                car.append(t.pop())
                if t:
                    stack_2.append(t)

            allocation[letter] = car

        pass
        print('machine ' + letter + 'loading completed！！！')
        print(allocation)  # print out results

    # file stores randomization results
    with open('allocation validation.txt', 'w', encoding='utf-8') as f:
        for i in range(ord('A'), ord('R') + 1):
            letter = chr(i)
            # for letter in string.ascii_uppercase:
            f.write(letter + ': ' + str(allocation[letter]))
            f.write('\n')

    return allocation

def Application(fertnames, combinations, allocation):

    print(fertnames)
    application = dict()
    for fert in fertnames:
        for i in range(3):
            application[fert + '_' + str(i)] = []
    application['Control'] = []

    keys = application.keys()
    comb_names = combinations.keys()

    for i in range(ord('A'), ord('R') + 1):
        letter = chr(i)
        i = 0
        for pod in allocation[letter]:
            i += 1
            if pod in keys:
                application[pod].append(letter + str(i))
            elif pod in comb_names:
                for fert in combinations[pod]:
                    application[fert].append(letter + str(i))

    with open('application validation.txt', 'w', encoding='utf-8') as f:
        for key in keys:
            f.write(key + ': ' + str(application[key]))
            f.write('\n')

    return application

if __name__ == "__main__":
    [fernames, combinations] = OACD_read('WisDM Green Experiment Design.xlsx')
    allocation = Allocation('WisDM Green Experiment Design.xlsx')
    application = Application(fernames, combinations, allocation)
    print(application)

```